CONVERTER_TYPE_ANY = 'any'
CONVERTER_TYPE_INT = 'int'
CONVERTER_TYPE_FLOAT = 'float'
CONVERTER_TYPE_UUID = 'uuid'
CONVERTER_TYPE_PATH = 'path'
CONVERTER_TYPE_STRING = 'string'
CONVERTER_TYPE_DEFAULT = 'default'
